MTO_example1(1);
function MTO_example1(exampleNumber)
fprintf(['                    ---------------------                  \n' ...
    'This is supplementary material provided with the publication \n' ...
    '"T. Kumar and K. Suresh (2019), A density-and-strain based K-clustering \n'...
    '    approach to microstructural topology optimization, submitted to Structural and Multidisciplinary Optimization"' ...
    '\n        Please cite the publication if using this resource\n'...
    '                    ---------------------                   \n\n']);
if nargin<1
    exampleNumber = 1;
end

% Each example is associated with a default set of parameters, namely:
% probType, clusteringType and nClusters

%%%%%%%%%%%%%%
% probType : example problem number from paper
%            1 Zhang and Sun's problem
%            2 L-bracket problem
%            3 Cantilever problem
%
% clusteringType: 1=grids, 2=density, 3=density-and-strain
%
% nClusters: desired number of clusters (max 20)
%            fixed for problem 1
%            choice of 3,5,7,12 with grid type in L-bracket, free otherwise


switch exampleNumber
    case 1 % For example Fig. 15(b)
        disp('Now running cantilever with grid based 8(+1 full density) clusters which is example Fig. 15(b)');
        probType = 1;
        clusteringType = 0;% is hard coded in main program
        nClusters = 0; % is hard coded in main program
    case 2 % For example Fig. 16(c),16(f), 16(i)
        probType = 2; % LBracket
        clusteringType = 3; % options: 1= grids, 2=density, 3=density-and-strain
        nClusters = 12;  % for grid-based clustering, enter 1,3,5,7, or 12, for the other two, enter between 1 and 20
    case 3 % For example Fig. 19(c-f)
        probType = 3; % cantilever
        clusteringType = 3; % options: 2=density, 3=density-and-strain
        nClusters = 10;  %  enter between 1 and 20
    case 4 % For example Fig. 19(c-f)
        probType = 4; % cantilever
        clusteringType = 3; % options: 2=density, 3=density-and-strain
        nClusters = 8;  %  enter between 1 and 20
    otherwise
        disp('Invalid option');
        return;
end

% Main program being called
main(probType,nClusters,clusteringType);
end
function main(probType,nClusters,clusteringType)
%% Inputs
% probType : example problem number from paper
%            1 Zhang and Sun's problem
%            2 L-bracket problem
%            3 Cantilever problem
% nClusters: desired number of clusters (max 20)
%            fixed for problem 1
%            choice of 3,5,7,12 with grid type in L-bracket, free otherwise
% clusteringType: 1=grids, 2=density, 3=density-and-strain
% This function minimizes compliance with respect volume
% clusteringType: grids, density, density-and-strain
% conduct SIMP (p=1) optimization and compute principal strain
% This function minimizes compliance with respect volume
if (nargin<1)
    probType = 1;
    nClusters = 0;
    clusteringType = 1;
elseif (nargin<2)
    nClusters = 12;
    clusteringType = 3;
elseif (nargin<3)
    clusteringType = 3;
end
% if probType ~=5
volConst = 2;
% else
%     volConst = 1;
% end
% volConst: 1=local volume constraint 2=global volume constraint (default)
nClusters = min(20,max(1,ceil(nClusters)));
switch probType
    case 1
        fileName = 'Validation.dat';
        nClusters = 10;
        clusteringType = 1;
        volFrac = 0.6;
        zoom = 150;
    case 2
        fileName = 'L_Bracket.dat';
        volFrac = 0.5;
        zoom = 33;
    case 3
        fileName = 'Cantilever.dat';
        volFrac = 0.5;
        zoom = 100;
    case 4
        fileName = 'bridge.dat';
        volFrac = 0.5;
        zoom = 100;
end
switch clusteringType
    case 1
        fprintf('%s\n','grid based clustering');
    case 2
        fprintf('%s\n','density based clustering');
    case 3
        fprintf('%s\n','density-and-strain based clustering');
end
move = 0.5;
dof = 2;
[elemData,nodeData,matProp,dispBC,forceBC,nNodes,nElems,~] = readData( fileName,dof);

xmax = max(nodeData(:,2));  ymax = max(nodeData(:,3));
xs = unique(nodeData(:,2)); ys = unique(nodeData(:,3));
nelx = length(xs)-1;        nely = length(ys)-1;
dx = xmax/nelx;             dy = ymax/nely;

t = matProp(1,4);

allDofs = (1:dof*nNodes)';
allNodes = (1:nNodes)';
mapNodesOldToNew(nodeData(:,1),1) = allNodes;
elemDataNew = [elemData(:,1:2),mapNodesOldToNew(elemData(:,3:6))];
elemData = elemDataNew;
nodeDataNew = [allNodes,nodeData(:,2:3)];
nodeData = nodeDataNew;
dispBC = [mapNodesOldToNew(dispBC(:,1)),dispBC(:,2:end)];
forceBC = [mapNodesOldToNew(forceBC(:,1)),forceBC(:,2:end)];

figure(1);
clf;
hold on
vx = reshape(nodeData(elemData(:,3:6)',2),4,[]);
vy = reshape(nodeData(elemData(:,3:6)',3),4,[]);
patch(vx,vy,1);
for i = 1:size(dispBC,1)
    plot(nodeData(dispBC(i,1),2),nodeData(dispBC(i,1),3),'b*');
end
for i = 1:size(forceBC,1)
    plot(nodeData(forceBC(i,1),2),nodeData(forceBC(i,1),3),'gd');
end
hold off

elemDofs=[2*nodeData(elemData(:,3),1)-1  2*nodeData(elemData(:,3),1) ...
    2*nodeData(elemData(:,4),1)-1 2*nodeData(elemData(:,4),1) ...
    2*nodeData(elemData(:,5),1)-1 2*nodeData(elemData(:,5),1) ...
    2*nodeData(elemData(:,6),1)-1 2*nodeData(elemData(:,6),1) ];

elemCenters = zeros(nElems,2);
elemCenters(:,1)= mean([nodeData(elemData(:,3),2) ...
    nodeData(elemData(:,4),2) nodeData(elemData(:,5),2) ...
    nodeData(elemData(:,6),2)],2);
elemCenters(:,2)= mean([nodeData(elemData(:,3),3) ...
    nodeData(elemData(:,4),3) nodeData(elemData(:,5),3) ...
    nodeData(elemData(:,6),3)],2);

prscdofs = unique((nodeData(dispBC(:,1),1)-1)*2+dispBC(:,2));
freeDofs=setdiff(allDofs,prscdofs);
[B,Det] = calc_Bdet1(elemData,nodeData);
elemList = (1:nElems)';
rho_vol=ones(size(elemData,1),1)*volFrac;
[Ke1,D] = calc_elem_mat(elemData,matProp,B,Det);
%Testing the hypothesis of using independent K components
Ke_unit(:,:,1) = Kelem_common_simple([1,0,0;0,0,0;0,0,0],t,B,Det{1,1});
Ke_unit(:,:,2) = Kelem_common_simple([0,0,0;0,1,0;0,0,0],t,B,Det{1,1});
Ke_unit(:,:,3) = Kelem_common_simple([0,0,0;0,0,0;0,0,1],t,B,Det{1,1});
Ke_unit(:,:,4) = Kelem_common_simple([0,1,0;1,0,0;0,0,0],t,B,Det{1,1});
Ke_unit(:,:,5) = Kelem_common_simple([0,0,1;0,0,0;1,0,0],t,B,Det{1,1});
Ke_unit(:,:,6) = Kelem_common_simple([0,0,0;0,0,1;0,1,0],t,B,Det{1,1});

iK = reshape(kron(elemDofs,ones(8,1))',64*nElems,1);
jK = reshape(kron(elemDofs,ones(1,8))',64*nElems,1);

volPerElem = dx*dy*ones(nElems,1);
min_rho = 0.001;

principalStrainDirectionOriginal = zeros(nElems,1);
switchTheseIndices = false(nElems,1);
maxIterations = 500;
%Get Initial density distribution usign simp p = 1 to help optimizer
if clusteringType >1
    interpolation = 1;
    penalty = 1;
    [~,objective,grad_macro] = solveLinearElasticityWithGradEfficient(Ke1,nodeData,rho_vol,penalty,freeDofs,forceBC,elemDofs,nNodes,iK,jK,interpolation);
    for iter = 1:maxIterations
        [rho_vol,change] = updateRho(rho_vol,grad_macro,volFrac,nElems,1e-3);
        if ~rem(iter,10)
            disp(['Iter. = ' num2str(iter) ', obj = ' num2str(objective) ', change = ' num2str(change) ', vol = ' num2str(sum(rho_vol.*volPerElem))]);
        end
        if(change<0.001)
            disp('Change in density converged for (p=1) SIMP');
            break;
        end
        [~,objective,grad_macro] = solveLinearElasticityWithGradEfficient(Ke1,nodeData,rho_vol,penalty,freeDofs,forceBC,elemDofs,nNodes,iK,jK,interpolation);
    end
end

XX(:,1) =  nodeData(elemData(1,3:6),2); XX(:,2) =  nodeData(elemData(1,3:6),3);
B0= B_Det(XX,0,0);

if clusteringType ==3
    u = solveLinearElasticityWithGradEfficient(Ke1,nodeData,rho_vol,penalty,freeDofs,forceBC,elemDofs,nNodes,iK,jK,interpolation);
    strain = cell2mat(get_strain_for_SIMP(u,B0,elemList',elemDofs));
    principalStrainDirection = atan2(strain(3,:),(strain(1,:)-strain(2,:)))/2+pi;
    principalStrainDirection = componentLabelling(nElems,nely,principalStrainDirection);
    pr_dir = reshape(principalStrainDirection,1,1,[]);
    ct = cos(pr_dir);st = sin(pr_dir);
    principalStrain = squeeze(sum([ct.^2 st.^2 ct.*st;st.^2 ct.^2 -ct.*st;-2*ct.*st 2*ct.*st ct.^2-st.^2].*repmat(reshape(strain(:),1,3,[]),3,1,1),2));
    switchTheseIndices = abs(principalStrain(1,:))<abs(principalStrain(2,:));
    principalStrainDirectionOriginal = principalStrainDirection;
    principalStrainDirection(switchTheseIndices) = principalStrainDirection(switchTheseIndices) - pi/2;
    temp = principalStrain(2,switchTheseIndices);
    principalStrain(2,switchTheseIndices) = principalStrain(1,switchTheseIndices);
    principalStrain(1,switchTheseIndices) = temp;
end

mi_nelx = 40;
mi_nely = 40;
[mi_iK,mi_jK,mi_iF,mi_jF,mi_nNodes,mi_nElems,mi_B,mi_Det,...
    mi_elemDofs,mi_newNodeData,mi_elemData,mi_freeDofs,mi_prscDofs,...
    mi_prscComp,mi_kfLogic]=generateMicrostructureFramework(mi_nelx,mi_nely,dx,dy);
mi_V_elem = (dx*dy)/(mi_nelx*mi_nely)*t;
mi_dx = dx/mi_nelx;
mi_dy = dy/mi_nely;

%define filter for micro-sensitivity
radius_as_numOfElem = 4;
mi_wgt = getPeriodicDomainWeight(mi_dx,mi_dy,mi_nElems,mi_nely,mi_V_elem/t,radius_as_numOfElem);

mi_rho = zeros(mi_nElems,nClusters);
zones = cell(nClusters,1);
obj = zeros(maxIterations,1);
change = zeros(maxIterations,1);
p1 = 3;

elemToGroupMapping = zeros(nElems,1);
for cluster = 1:nClusters
    elemToGroupMapping(and(rho_vol>(cluster-1)/(nClusters),rho_vol<=cluster/(nClusters))) = cluster;
end

if clusteringType == 3
    criteria = [rho_vol';((principalStrain(2,:).^2)./principalStrain(1,:).^2)];
    criteria(:,rho_vol<0.1) = -5;
    criteria(:,rho_vol>0.9) = 5;
elseif clusteringType == 2
    criteria = rho_vol';
elseif clusteringType == 1
    if probType == 1
        nDivX = 1;nDivY = 10;
        criteria = zeros(1,nElems);
        criteria = criteria + floor((elemCenters(:,1)'/xmax)/(1/nDivX));
        criteria = criteria + nDivX*floor((elemCenters(:,2)'/ymax)/(1/nDivY));
        criteria(criteria == min(criteria)) = -50;
        criteria(criteria == max(criteria)) = -50;
        nClusters = 9;
    elseif probType == 3
        nDivX = 5;nDivY = 2;
        criteria = zeros(1,nElems);
        criteria = criteria + floor((elemCenters(:,1)'/xmax)/(1/nDivX));
        criteria = criteria + nDivX*floor((elemCenters(:,2)'/ymax)/(1/nDivY));
    elseif probType == 4
        nDivX = 4;nDivY = 2;
        criteria = zeros(1,nElems);
        criteria = criteria + floor((elemCenters(:,1)'/xmax)/(1/nDivX));
        criteria = criteria + nDivX*floor((elemCenters(:,2)'/ymax)/(1/nDivY));
    else
        if nClusters == 12
            nDivX = 4;nDivY = 4;
            Xdiv = [0,20,40,70];
        elseif nClusters ==3
            nDivX = 2;nDivY = 2;
            Xdiv = [0,40];
        elseif nClusters ==5
            nDivX = 3;nDivY = 3;
            Xdiv = [0,40,70];
        elseif nClusters == 1
            nDivX = 1;nDivY = 1;
            Xdiv = 0;
        else
            disp(['Error: cannot divide L-bracket into ' num2str(nClusters) ' Clusters']);
            return;
        end
        Ydiv = Xdiv;
        criteria = zeros(1,nElems);
        criteria = criteria + (sum(repmat(elemCenters(:,1),1,nDivX)>repmat(Xdiv,nElems,1),2))';
        criteria = criteria + (sum(repmat(elemCenters(:,2),1,nDivY)>repmat(Ydiv,nElems,1),2)*nDivX)';
    end
end
if clusteringType ~=0
    elemToGroupMapping = myKmeans(criteria,nClusters,1);
else
    nClusters = nElems;
    elemToGroupMapping = (1:nElems)';
end

for cluster = 1:nClusters
    zones{cluster,1} = elemList(elemToGroupMapping==cluster,1);
end

volAssignedPerGroup = zeros(nClusters,1);
if probType == 1
    rho_vol(zones{1,1},1) =1;
    rho_vol(setdiff(1:nElems,zones{1,1}),1) = (nElems*volFrac-numel(zones{1,1}))/(nElems-numel(zones{1,1}));
elseif probType == 4
    rho_vol(zones{2,1},1) = 1;
    rho_vol(zones{1,1},1) = (volFrac*nElems-numel(zones{2,1}))/numel(zones{1,1});
end
grad_wrt_mi_rho = zeros(mi_nElems,nClusters);
for cluster = 1:nClusters
    meanRho = mean(rho_vol(zones{cluster,1},:));
    volAssignedPerGroup(cluster,1) = mean(rho_vol(zones{cluster,1},:));
    mi_rho(:,cluster) = getInitialMicroDesign(mi_elemData,mi_newNodeData,meanRho,min_rho,0.2,1);
end
if probType == 1
    mi_rho(:,1) = 1;
    mi_rho = mi_rho(:,1:nClusters);
    zones = zones(1:nClusters,1);
elseif probType == 4
    mi_rho(:,2) = 1;
end
volPerMicroElem = (dx*dy)/(mi_nelx*mi_nely)*repmat(cellfun(@(x) numel(x),zones)',mi_nElems,1);
disp(['initial volume fraction = ' num2str(sum(sum(mi_rho.*volPerMicroElem))/sum(sum(volPerMicroElem)))]);
totVolume = sum(sum(volPerMicroElem));

if clusteringType == 3
    theta =-principalStrainDirection';
else
    theta = zeros(nElems,1);%
end
%Precomputing Q in a large sparse rotation matrix
iQ = kron(ones(nElems,1),repmat([1;2;3],3,1))+kron((0:(nElems-1))'*3,ones(9,1));
jQ = kron((1:3*nElems)',ones(3,1));
sQ = zeros(9*nElems,1);

for  elem = 1:nElems
    ct = cos(theta(elem));
    st = sin(theta(elem));
    c2t = cos(2*theta(elem));
    s2t = sin(2*theta(elem));
    T = [ct^2 st^2 s2t;st^2 ct^2 -s2t;-s2t/2 s2t/2 c2t];
    sQ((elem-1)*9+1:elem*9)= T(:);
end
Q = sparse(iQ,jQ,sQ);

vol = zeros(maxIterations,1);
currentVolume = sum(sum(mi_rho.*volPerMicroElem));
vol(1) = currentVolume;

for iter = 1:maxIterations
    [micro_u]=solveMicrostructureFEA_efficient(mi_rho,D,mi_iK,mi_jK,mi_iF,mi_jF,mi_nNodes,t,p1,mi_B,mi_Det,mi_freeDofs,mi_prscDofs,mi_prscComp,mi_kfLogic);
    [D_homo,grad_D] = getHomogenizedDandGradient(micro_u,mi_nElems,D,mi_B,mi_Det,t,mi_elemDofs,mi_rho,mi_V_elem,p1);
    [objective,grad_wrt_mi_rho_elem] = solLinElastWithGlobalGrad_MicroStructRotatEffi(D_homo,Ke_unit,nodeData,t,rho_vol,theta,freeDofs,forceBC,elemDofs,nNodes,B,Det,grad_D,zones,Q,iQ,jQ);
    
    for cluster = 1:nClusters
        grad_wrt_mi_rho(:,cluster) = sum(grad_wrt_mi_rho_elem(:,elemToGroupMapping == cluster),2);
    end
    obj(iter) = objective;
    
    grad_wrt_mi_rho = (mi_wgt*(mi_rho.*grad_wrt_mi_rho./volPerMicroElem)).*(volPerMicroElem./mi_rho);
    if volConst == 1
        for cluster = 1:nClusters
            [mi_rho(:,cluster),changeCtr] = updateRho1(mi_rho(:,cluster),grad_wrt_mi_rho(:,cluster),...
                volAssignedPerGroup(cluster,1),sum(volPerMicroElem(:,cluster)),min_rho,volPerMicroElem(:,cluster),1e-8,move,0.5);
            change(iter) = max(changeCtr,change(iter));
        end
    else
        if probType == 1
            [mi_rho,change(iter)] = updateRho1(mi_rho,grad_wrt_mi_rho,...
                volFrac,totVolume,min_rho,volPerMicroElem,1e-8,move,0);
            mi_rho(:,1) = 1;
        else
            [mi_rho,change(iter)] = updateRho1(mi_rho,grad_wrt_mi_rho,...
                volFrac,totVolume,min_rho,volPerMicroElem,1e-8,move,1);
        end
    end
    currentVolume = sum(sum(mi_rho.*volPerMicroElem));
    vol(iter+1) = currentVolume;
    disp(['Iter. = ' num2str(iter) ', obj = ' num2str(obj(iter)) ', change = ' num2str(change(iter)) ', vol = ' num2str(currentVolume)]);
    if iter>2
        if((abs((obj(iter)-obj(iter -1))/obj(iter-1))<5e-6) && (abs((obj(iter-1)-obj(iter -2))/obj(iter-2))<5e-6))&&radius_as_numOfElem==1
            disp('Objective Function Converged');
            break;
        elseif((abs((obj(iter)-obj(iter -1))/obj(iter-1))<1e-4) && (abs((obj(iter-1)-obj(iter -2))/obj(iter-2))<1e-4))
            radius_as_numOfElem = max(radius_as_numOfElem-0.5,1);
            mi_wgt = getPeriodicDomainWeight(mi_dx,mi_dy,mi_nElems,mi_nely,mi_V_elem/t,radius_as_numOfElem);
        end
    end
    if(change(iter)<0.005&&radius_as_numOfElem==1)
        disp('Change in density converged');
        break;
    end
end
prin_nelx = 40;prin_nely = 40;
if clusteringType == 3
    multiplier = 10;
    p_elx = prin_nelx*multiplier; p_ely = prin_nely*multiplier;
    pixelImageData= printMorphedDomain(p_elx, p_ely,mi_rho,elemData,nodeData,elemToGroupMapping,switchTheseIndices,nElems,nNodes,principalStrainDirectionOriginal,dx,dy,nely,nelx,mi_nely,mi_nelx,t,forceBC(1+floor(size(forceBC,1)/2),1));
    newPID = zeros(prin_nely*nely,prin_nelx*nelx);
    for i = multiplier:multiplier:size(pixelImageData,1)
        for j = multiplier:multiplier:size(pixelImageData,2)
            newPID(i/multiplier,j/multiplier) = sum(sum(pixelImageData(i-multiplier+1:i,j-multiplier+1:j)))/multiplier^2;
        end
    end
    pixelImageData = newPID;
else
    pixelImageData= ones(prin_nely*nely,prin_nelx*nelx);
    for elem = 1:nElems
        Idx = fliplr(round(nodeData(elemData(elem,3),2:3)/dx)).*[prin_nely,prin_nelx];
        pixelImageData(Idx(1)+1:Idx(1)+prin_nely,Idx(2)+1:Idx(2)+prin_nelx)...
            = 1-reshape(mi_rho(:,elemToGroupMapping(elem)),mi_nely,mi_nelx);
    end
end
figure();imshow(pixelImageData,'InitialMagnification',zoom);colormap('gray');axis off;axis equal;
axis xy
save(['prob' num2str(probType) 'C' num2str(nClusters) 't' num2str(clusteringType) '.mat'],'pixelImageData');
finalAnalysis(probType,3,['prob' num2str(probType) 'C' num2str(nClusters) 't' num2str(clusteringType) '.mat']);
end

function finalAnalysis(pblm,penal,filename)
%% MATERIAL PROPERTIES
load(filename,'pixelImageData');
[nely,nelx] = size(pixelImageData);
Emin = 0;
nu = 0.3;
%% PREPARE FINITE ELEMENT ANALYSIS
A11 = [12  3 -6 -3;  3 12  3  0; -6  3 12 -3; -3  0 -3 12];
A12 = [-6 -3  0  3; -3 -6 -3 -6;  0 -3 -6  3;  3 -6  3 -6];
B11 = [-4  3 -2  9;  3 -4 -9  4; -2 -9 -4 -3;  9  4 -3 -4];
B12 = [ 2 -3  4 -9; -3  2  9 -2;  4  9  2  3; -9 -2  3  2];
KE = 1/(1-nu^2)/24*([A11 A12;A12' A11]+nu*[B11 B12;B12' B11]);
if pblm == 2
    E0 = 10;
    nely1 = nely; nely2 = nely*2/5;nelx1 = nelx*2/5;nelx2 = nelx*3/5;
    nElems = nelx1*nely1+nelx2*nely2;
    nodenrs1 = reshape(1:(1+nelx1)*(1+nely1),1+nely1,1+nelx1);
    nodenrs3 = reshape(nodenrs1(end,end)+(1:(nelx2)*(1+nely2)),1+nely2,nelx2);
    nodenrs2 = [nodenrs1(1:(nely2+1),end),nodenrs3(:,1)];
    nNodes = numel([nodenrs1(:);nodenrs3(:)]);
    edofVec1 = 2*reshape(nodenrs1(1:end-1,1:end-1),[],1)+1;
    edofVec2 = 2*reshape(nodenrs2(1:end-1,1:end-1),[],1)+1;
    edofVec3 = 2*reshape(nodenrs3(1:end-1,1:end-1),[],1)+1;
    edofMat1 = repmat(edofVec1,1,8)+repmat([0 1 2*nely1+[2 3 0 1] -2 -1],size(edofVec1,1),1);
    edofMat2 = repmat(edofVec2,1,8)+repmat([0 1 2*nely1+[2 3 0 1] -2 -1],size(edofVec2,1),1);
    edofMat3 = repmat(edofVec3,1,8)+repmat([0 1 2*nely2+[2 3 0 1] -2 -1],size(edofVec3,1),1);
    edofMat = [edofMat1;edofMat2;edofMat3];
    
    %Force for L-bracket
    F = sparse(2*nodenrs3(end,end),1,10,2*nNodes,1);
    
    fixeddofs = [2*nodenrs1(end,:),2*nodenrs1(end,:)-1];
    
    x = 1-pixelImageData;
    xPhys = [reshape(x(1:end,1:nelx1),[],1);reshape(x(1:nely2,(nelx1+1):end),[],1)];
    
else
    if pblm == 1
        E0 = 1000;
    else
        E0 = 2.1e5;
    end
    x = 1-pixelImageData;
    xPhys = x(:);
    nodenrs = reshape(1:(1+nelx)*(1+nely),1+nely,1+nelx);
    edofVec = reshape(2*nodenrs(1:end-1,1:end-1)+1,nelx*nely,1);
    edofMat = repmat(edofVec,1,8)+repmat([0 1 2*nely+[2 3 0 1] -2 -1],nelx*nely,1);
    nNodes = (1+nelx)*(1+nely);
    nElems = nelx*nely;
    % DEFINE LOADS AND SUPPORTS (HALF MBB-BEAM)
    % Force for cantilever problem
    if pblm == 1
        F = sparse(2*(nely+1)*(nelx+1)-(nely)+(-2*200:2:2*200),1,2000/(2*200),2*(nely+1)*(nelx+1),1);
        F(2*(nely+1)*(nelx+1)-(nely)+[-2*200,2*200]) = 2000/(2*200)/2;
        fixeddofs = 1:2*(nely+1);
        xPhys((nely)*(nelx)-(nely/2)+(-200:200)) = 1;
    elseif pblm == 3
        F = sparse(2*(nely+1)*(nelx+1) - nely +(-2*floor(nely/20):2:2*ceil(nely/20)),1,-200/(2*(nely/20)),2*(nely+1)*(nelx+1),1);
        F(2*(nely+1)*(nelx+1) - nely -2*floor(nely/20),1) = -200/(4*(nely/20));
        F(2*(nely+1)*(nelx+1) - nely +2*ceil(nely/20),1) = -200/(4*(nely/20));
        fixeddofs = 1:2*(nely+1);
        xPhys((nely)*(nelx)-(nely/2)+(-80:80)) = 1;
    elseif pblm == 4
        F = sparse(2,1,-1000,2*(nely+1)*(nelx+1),1);
        fixeddofs = [1:2:2*(nely+1),2*(nely+1)*nelx+2];
        xPhys((nely)*(nelx)-(nely/2)+(-80:80)) = 1;
    end
end
iK = reshape(kron(edofMat,ones(8,1))',64*nElems,1);
jK = reshape(kron(edofMat,ones(1,8))',64*nElems,1);

U = zeros(2*nNodes,1);
alldofs = 1:2*nNodes;
freedofs = setdiff(alldofs,fixeddofs);
%% INITIALIZE ITERATION
xPhys(xPhys<0.001) = 0.001;
%% FE-ANALYSIS
sK = reshape(KE(:)*(Emin+xPhys(:)'.^penal*(E0-Emin)),64*nElems,1);
K = sparse(iK,jK,sK); K = (K+K')/2;
U(freedofs) = K(freedofs,freedofs)\F(freedofs);
%% OBJECTIVE FUNCTION AND SENSITIVITY ANALYSIS
ce = reshape(sum((U(edofMat)*KE).*U(edofMat),2),nElems,1);
xPhysCe = (Emin+xPhys.^penal*(E0-Emin)).*ce;
c = sum(xPhysCe);
disp(['Fine Scale Analysis: ' num2str(loop) 'compliance = ' num2str(c) ' volFrac = ' num2str(sum(sum(xPhys))/nElems)]);
end
% function D=getD(YM,nu)
% D=zeros(3,3);
% dummy = YM/(1-nu*nu);
% D(1,1) = dummy;
% D(1,2) = dummy*nu;
% D(2,1) = D(1,2);
% D(2,2) = D(1,1);
% D(3,3) = dummy*(1-nu)/2;
% end

function principalStrainDirection = componentLabelling(nElems,nely,principalStrainDirection)
loop = 0;
while(1)
    loop = loop +1;
    neighbors = num2cell(bsxfun(@plus,(1:nElems)',[-nely,-1,1,nely]));
    componentOfElem = zeros(nElems,1);
    currentComponent = 1;
    labelCounter = 0;
    Queue = cell(1,1);
    
    for elem = 1:nElems
        if componentOfElem(elem) == 0
            Queue{1,1} = [Queue{1,1},elem];
            componentOfElem(elem,1) = currentComponent;
            currentComponent = currentComponent + 1;
            labelCounter = labelCounter + 1;
            
            while(~isempty(Queue{1,1}))
                thisElem = Queue{1,1}(end);
                Queue{1,1} = Queue{1,1}(1:end-1);
                for neighborsIndex = 1:4
                    switch rem(thisElem,nely)
                        case 1
                            if neighborsIndex == 2
                                continue;
                            end
                        case 0
                            if neighborsIndex == 3
                                continue;
                            end
                    end
                    neighbor = neighbors{thisElem,neighborsIndex};
                    if neighbor < 1 || neighbor >nElems
                        continue
                    end
                    if componentOfElem(neighbor) == 0
                        if abs(principalStrainDirection(neighbor)-principalStrainDirection(thisElem))<pi/2
                            componentOfElem(neighbor) = componentOfElem(thisElem,1);
                            Queue{1,1} = [Queue{1,1},neighbor];
                            labelCounter = labelCounter + 1;
                        end
                    end
                end
                if labelCounter == nElems
                    break;
                end
            end
        end
    end
    if currentComponent == 2
        break;
    end
    numberOfComponentsPerLabel = zeros(currentComponent-1,1);
    for comp = 1:currentComponent-1
        numberOfComponentsPerLabel(comp,1) = sum(componentOfElem == comp);
    end
    [~,maxCompnent] = max(numberOfComponentsPerLabel);
    for comp = 1:currentComponent-1
        if comp == maxCompnent
            continue;
        end
        avg = mean(principalStrainDirection(componentOfElem == comp));
        if avg>pi
            principalStrainDirection(componentOfElem == comp) = principalStrainDirection(componentOfElem == comp) - pi;
        else
            principalStrainDirection(componentOfElem == comp) = principalStrainDirection(componentOfElem == comp) + pi;
        end
    end
end
end

function pixelImageData= printMorphedDomain(prin_nelx, prin_nely,mi_rho,elemData,nodeData,elemToGroupMapping,switchTheseIndices,nElems,nNodes,principalStrainDirectionOriginal,dx,dy,nely,nelx,mi_nely,mi_nelx,t,forceNode)
phis = getMorphedCoordinates1(elemData,nElems,nNodes,principalStrainDirectionOriginal,dx,dy,t,nelx,nely,nodeData,sum(mi_rho(:,elemToGroupMapping),1)/(mi_nely*mi_nelx),forceNode);
pixelImageData = printUsingPhi(phis,prin_nelx,prin_nely,1,1,elemData,nodeData,elemToGroupMapping,switchTheseIndices,dx,dy,mi_rho,nely,nelx,mi_nely,mi_nelx,nElems,1:nElems,1:nNodes);
end

function pixelImageData = printUsingPhi(phis,prin_nelx,prin_nely,mult_x,mult_y,elemData,nodeData,elemToGroupMapping,switchTheseIndices,dx,dy,mi_rho,nely,nelx,mi_nely,mi_nelx,nActiveElems,activeElems,allActiveNodes)
phi_x = phis(:,1);phi_y = phis(:,2);
%Grid for printing
[prin_xi,prin_eta] = meshgrid(linspace(-1,1,prin_nely+1),linspace(-1,1,prin_nelx+1));
offsetX = phi_x+max(max(-phi_x));
offsetY = phi_y+max(max(-phi_y));
clearvars phi_x phi_y phis
offsetXScaled = zeros([prin_nely,prin_nelx].*[nely,nelx]+[1,1]);
offsetYScaled = zeros([prin_nely,prin_nelx].*[nely,nelx]+[1,1]);
for elem = 1:nActiveElems
    thisElem = activeElems(elem);
    j = round(nodeData(elemData(thisElem,3),2)/dx)+1;
    i = round(nodeData(elemData(thisElem,3),3)/dy)+1;
    currentXs = offsetX(allActiveNodes(elemData(thisElem,3:6)));
    offsetXScaled((i-1)*prin_nely+1:i*prin_nely+1,(j-1)*prin_nelx+1:j*prin_nelx+1)...
        = interpolateValuesInTheInterior(currentXs,prin_xi,prin_eta);
    currentYs = offsetY(allActiveNodes(elemData(thisElem,3:6)));
    offsetYScaled((i-1)*prin_nely+1:i*prin_nely+1,(j-1)*prin_nelx+1:j*prin_nelx+1)...
        = interpolateValuesInTheInterior(currentYs,prin_xi,prin_eta);
end
clearvars prin_eta prin_xi
elemPosition = nodeData(elemData(:,3),2:3);
elemPositionGrid = round([elemPosition(:,1)/dx,elemPosition(:,2)/dy]);
clearvars nodeData elemData
mi_dx = dx/mi_nelx;mi_dy = dy/mi_nely;
pixelImageData = ones(max(elemPositionGrid(:,2)+1)*prin_nely,max(elemPositionGrid(:,1)+1)*prin_nelx);
clearvars  elemPositionGrid
offsetXScaled = (offsetXScaled(1:end-1,:)+ offsetXScaled(2:end,:))/2;
offsetXScaled = (offsetXScaled(:,1:end-1)+ offsetXScaled(:,2:end))/2;
offsetYScaled = (offsetYScaled(1:end-1,:)+ offsetYScaled(2:end,:))/2;
offsetYScaled = (offsetYScaled(:,1:end-1)+ offsetYScaled(:,2:end))/2;
pixelCtr = 0;
[minX]  = min(offsetXScaled(:));
Xd = minX-rem(minX,dx/mult_x);
allIdx = nan(numel(offsetXScaled),1);
allEntries = ones(numel(offsetXScaled),1);
height = size(pixelImageData,1);
while(pixelCtr<numel(offsetXScaled))
    Xu = Xd + dx/mult_x;
    currStripIdx = find(and(offsetXScaled>=Xd,offsetXScaled<Xu));
    sizeOfStrip = length(currStripIdx);%nnz(currStrip);
    [minY] = min(min(offsetYScaled(currStripIdx)));
    pixelCtrY = 0;
    Yd = minY-rem(minY,dy/mult_y);
    while(pixelCtrY<sizeOfStrip)
        Yu = Yd + dy/mult_y;
        thisBoxIdx = currStripIdx(and(offsetYScaled(currStripIdx)>=Yd,offsetYScaled(currStripIdx)<Yu));
        sizeOfBox = length(thisBoxIdx);
        row = rem(thisBoxIdx-1,height)+1;
        col = floor((thisBoxIdx-1)/height)+1;
        Yd = Yu;
        allIdx(pixelCtr+pixelCtrY+1:pixelCtr+pixelCtrY+sizeOfBox) = thisBoxIdx;
        pixelCtrY = pixelCtrY + sizeOfBox;
        center = [mean(row)/prin_nely*dy,mean(col)/prin_nelx*dx];
        [~,idx] = min((elemPosition(:,2)+dy/2-center(1)).^2+(elemPosition(:,1)+dx/2-center(2)).^2);
        if (center(1)>=elemPosition(idx,2)&&center(1)<=elemPosition(idx,2)+dy)&&(center(2)>=elemPosition(idx,1)&&center(2)<=elemPosition(idx,1)+dx)
            elem = idx;
        else
            continue;
        end
        zoneOfThisElem = elemToGroupMapping(elem);
        switch zoneOfThisElem
            case 1
                rho = 0.999;
                allEntries(pixelCtr+pixelCtrY-sizeOfBox+1:pixelCtr+pixelCtrY) = rho;
            case 2
                rho = 0;
                allEntries(pixelCtr+pixelCtrY-sizeOfBox+1:pixelCtr+pixelCtrY) = rho;
            otherwise
                xs = rem(offsetXScaled(thisBoxIdx),dx/mult_x);
                ys = rem(offsetYScaled(thisBoxIdx),dy/mult_y);
                centerIdx_col = max(ceil(xs*mult_x/mi_dx),1);
                centerIdx_row = max(ceil(ys*mult_y/mi_dy),1);
                if switchTheseIndices(elem)
                    thisElemRho = reshape(reshape(mi_rho(:,zoneOfThisElem),mi_nely,mi_nelx)',[],1);
                else
                    thisElemRho = mi_rho(:,zoneOfThisElem);
                end
                rho_new = thisElemRho(sub2ind([mi_nely,mi_nelx],centerIdx_row,centerIdx_col));
                rho = 1-rho_new;
                allEntries(pixelCtr+pixelCtrY-sizeOfBox+1:pixelCtr+pixelCtrY) = rho;
        end
    end
    pixelCtr = pixelCtr + pixelCtrY;
    Xd = Xu;
end
pixelImageData(allIdx) = allEntries;
figure(2);
imshow(pixelImageData,'InitialMagnification','fit');
axis xy
end

function [phis] = getMorphedCoordinates1(elemData,nElems,nNodes,theta,dx,dy,t,nelx,nely,nodeData,rho,forceNode)
thetaAtMacroNodes = zeros(nNodes,1);

parfor node = 1:4
    thetaAtMacroNodes = thetaAtMacroNodes + accumarray(elemData(:,2+node),theta',[nNodes,1]);
end
repeatOfNodes = accumarray(reshape(elemData(:,3:6),[],1),1);
thetaAtMacroNodes = thetaAtMacroNodes./repeatOfNodes;
clearvars repeatOfNodes
nSubDiv = 1;
thetaAtAllNodes = nan((nSubDiv+1),(nSubDiv+1),nElems);
thetaAtMacroNodesNew =  nan((nely+1),(nelx+1));
[xis,etas] = meshgrid(linspace(-1,1,nSubDiv+1),linspace(-1,1,nSubDiv+1));
for elem = 1:nElems
    nodalX = [-1,1,1,-1];
    nodalY = [-1,-1,1,1];
    nodalThetas = thetaAtMacroNodes(elemData(elem,3:6));
    thetaAtAllNodes(:,:,elem) = interpolateValuesInTheInterior(nodalThetas,xis,etas,nodalX,nodalY);
    elemY = nodeData(elemData(elem,6),3)/dy;
    elemX = nodeData(elemData(elem,4),2)/dx;
    thetaAtMacroNodesNew(((elemY-1)*(nSubDiv)+1):((elemY)*(nSubDiv)+1),((elemX-1)*(nSubDiv)+1):((elemX)*(nSubDiv)+1)) = thetaAtAllNodes(:,:,elem);
end
% Jacobian calculation
sampleX = [0;dx/nSubDiv;dx/nSubDiv;0];
sampleY = [0;0;dy/nSubDiv;dy/nSubDiv];
xis = 0;
etas =0;
[~,dN] = getShapeFunctionAndItsDerivative(xis,etas);
J = (dN'*[sampleX,sampleY])';
J1 = fliplr(J);
J1(:,1) = -J1(:,1);
J1inv = inv(J1);
detJ = det(J);
Jinv = inv(J);

%Stiffness matrix calculation
xis = [-1,1,1,-1]'/sqrt(3);
etas =[-1,-1,1,1]'/sqrt(3);
[~,dN] = getShapeFunctionAndItsDerivative(xis,etas);
THIC = t;
WGT = [1.0  1.0  1.0  1.0];
BTransB=zeros(4,4);
BTransBTilde=zeros(4,4);
% BTransE = zeros(4,1);
parfor i=1:4
    WT = WGT(i)*THIC;
    BTransB = BTransB + dN(:,(2*i-1):2*i)*Jinv'*Jinv*dN(:,(2*i-1):2*i)'*WT*detJ;
    BTransBTilde = BTransBTilde + dN(:,(2*i-1):2*i)*Jinv'*J1inv*dN(:,(2*i-1):2*i)'*WT*detJ;
end

nodeNumbers = nodeData(:,1);
edofMat = nodeNumbers(elemData(:,3:6));

weight = ones(nElems,1);
weight(rho<0.1) = 0.01;
iK = reshape(kron(edofMat,ones(4,1))',16*nElems,1);
jK = reshape(kron(edofMat,ones(1,4))',16*nElems,1);
kK = reshape(BTransBTilde(:)*weight',16*nElems,1);
K_global1 = sparse(iK,jK,kK,nNodes,nNodes);
kK = reshape(BTransB(:)*weight',16*nElems,1);
K_global2 = sparse(iK,jK,kK,nNodes,nNodes);
clearvars iK jK kK

nodalX = [-1,1,1,-1];
nodalY = [-1,-1,1,1];

F_global = K_global1*thetaAtMacroNodes;
all_inter_dofs = 1:nNodes;%,(nely*mi_inter_nely+1)*(nelx*mi_inter_nelx+1)+1];
fixed_inter_dofs  = forceNode;%[1:(nely*mi_inter_nely+1)];
free_inter_dofs = setdiff(all_inter_dofs,fixed_inter_dofs)';%,1:(nely*mi_inter_nely+1):(nely*mi_inter_nely+1)*(nelx*mi_inter_nelx+1)]);
r = zeros(length(all_inter_dofs),1);
r(free_inter_dofs,1)= K_global2(free_inter_dofs,free_inter_dofs)\F_global(free_inter_dofs);

xis = [-1,1,1,-1]'/sqrt(3);
etas =[-1,-1,1,1]'/sqrt(3);
[~,dN] = getShapeFunctionAndItsDerivative(xis,etas);
kF1 = zeros(nElems,4);
kF2 = zeros(nElems,4);
parfor elem = 1:nElems
    nodalThetas = thetaAtMacroNodes(edofMat(elem,:));
    nodalr = r(edofMat(elem,:));
    thetaAtGaussPt = interpolateValuesInTheInterior(nodalThetas,xis,etas,nodalX,nodalY);
    rAtGaussPt = interpolateValuesInTheInterior(nodalr,xis,etas,nodalX,nodalY);
    for i = 1:4
        WT = WGT(i)*THIC;
        E1 = exp(rAtGaussPt(i,1))*[cos(thetaAtGaussPt(i,1));sin(thetaAtGaussPt(i,1))];%
        E2 = exp(rAtGaussPt(i,1))*[-sin(thetaAtGaussPt(i,1));cos(thetaAtGaussPt(i,1))];%exp(rAtGaussPt(i,1))*
        kF1(elem,:) = kF1(elem,:) + weight(elem,1)*(dN(:,(2*i-1):2*i)*Jinv'*E1*WT*detJ)';%weight(elem,1)*
        kF2(elem,:) = kF2(elem,:) + weight(elem,1)*(dN(:,(2*i-1):2*i)*Jinv'*E2*WT*detJ)';%
    end
end

F_global1 = sparse(edofMat,1,kF1,nNodes,1);
F_global2 = sparse(edofMat,1,kF2,nNodes,1);
%clearvars kF1 kF2
all_inter_dofs = 1:nNodes;%,(nely*mi_inter_nely+1)*(nelx*mi_inter_nelx+1)+1];
fixed_inter_dofs  = forceNode;%[1:(nely*mi_inter_nely+1)];
free_inter_dofs = setdiff(all_inter_dofs,fixed_inter_dofs)';%,1:(nely*mi_inter_nely+1):(nely*mi_inter_nely+1)*(nelx*mi_inter_nelx+1)]);
%clearvars fixed_inter_dofs allActiveNodes thetaAtGaussPt thetaAtMacroNodes
phis = zeros(length(all_inter_dofs),2);
phis(free_inter_dofs,1)= K_global2(free_inter_dofs,free_inter_dofs)\F_global1(free_inter_dofs);
phis(free_inter_dofs,2)= K_global2(free_inter_dofs,free_inter_dofs)\F_global2(free_inter_dofs);
phi1 = phis(:,1);

idx = sub2ind([nely+1,nelx+1],nodeData(:,3)/dy+1,nodeData(:,2)/dx+1);
phi1Norm = sqrt(sum((phi1(edofMat(and(rho>0.1,rho<0.9),:))*BTransB).*phi1(edofMat(and(rho>0.1,rho<0.9),:)),2));
phi1 = phi1/mean(phi1Norm)*dx*2;
phi1rect = nan(nely+1,nelx+1);
phi1rect(idx) = phi1;
phi2 = phis(:,2);

phi2Norm = sqrt(sum((phi2(edofMat(and(rho>0.1,rho<0.9),:))*BTransB).*phi2(edofMat(and(rho>0.1,rho<0.9),:)),2));
phi2 = phi2/mean(phi2Norm)*dx*2;
phi2rect = nan(nely+1,nelx+1);
phi2rect(idx) = phi2;

figure(3);clf;
hold on;
contour(phi1rect,[0:-dx:min(phi1),dx:dx:max(phi1)],'r');
contour(phi2rect,[0:-dx:min(phi2),dx:dx:max(phi2)],'b');%,'ShowText','on'
quiver(reshape(cos(thetaAtMacroNodesNew),(nely+1),(nelx+1)),reshape(sin(thetaAtMacroNodesNew),(nely+1),(nelx+1)),'k');
hold off
axis([1 nelx+1 1 nely+1]);
drawnow
phis = [phi1,phi2];
end

function [elemToGroupMapping,C] = myKmeans(X,nGroups,distanceType)
[M,N] = size(X);
if nGroups == 1
    elemToGroupMapping = ones(1,N);
    C = mean(X,2);
    return;
end
%% kmeans ++
gIndex = zeros(nGroups,1);
[~,gIndex(1)] = min(sum(X,1));
[~,gIndex(2)] = max(sum(X,1));
%distanceType = 1; % 1= manhattan, 2 = euclidean
HUGE_NUMBER = 4*(max(max(abs(X)))*M)^2;
for group = 3:nGroups
    D = HUGE_NUMBER*ones(1,N);
    if distanceType == 1
        for i = 1:group-1
            D = min(sum(abs(X-repmat(X(:,gIndex(i,1)),1,N)),1),D);
        end
        
    elseif distanceType == 2
        for i = 1:group-1
            D = min(sum((X-repmat(X(:,gIndex(i,1)),1,N)).^2,1),D);
        end
    end
    prob = D.^2./sum(D.^2);
    [~,gIndex(group)] = max(prob);
end
%% kmeans
C = X(:,gIndex);
elemToGroupMapping = ones(1,N);
D = HUGE_NUMBER*ones(N,1);
change = 1;
while(change>0)
    change = 0;
    if distanceType == 1
        %D = sum(abs(X-C(:,elemToGroupMapping)),1);
        [newD,indices] = min(squeeze(sum(abs(repmat(X,1,1,nGroups)-repmat(reshape(C,M,1,[]),1,N,1)),1)),[],2);
        logicalIndices = newD<D;
        D(logicalIndices) = newD(logicalIndices);
        elemToGroupMapping(logicalIndices) = indices(logicalIndices);
        change = sum([change;logicalIndices]);
        
    elseif distanceType == 2
        %D = sum((X-C(:,elemToGroupMapping)).^2,1);
        [newD,indices] = min(squeeze(sum((repmat(X,1,1,nGroups)-repmat(reshape(C,M,1,[]),1,N,1)).^2,1)),[],2);
        logicalIndices = newD<D;
        D(logicalIndices) = newD(logicalIndices);
        elemToGroupMapping(logicalIndices) = indices(logicalIndices);
        change = sum([change;logicalIndices]);
    end
    for i = 1:nGroups
        C(:,i) = mean(X(:,elemToGroupMapping==i),2);
    end
end
end
function initial_mi_rho = getInitialMicroDesign(mi_elemData,mi_newNodeData,volFrac,min_rho,DiffInDensity,AR)
if nargin<5
    min_rho = 0.001;
    DiffInDensity = 0.2;
end
if nargin<8
    AR = 1;
end
xmax = max(mi_newNodeData(:,2));  ymax = max(mi_newNodeData(:,3));
dx = xmax;dy = ymax;
mi_nElems = size(mi_elemData,1);
mi_elemCenters = zeros(mi_nElems,2);
mi_elemCenters(:,1)= mean([mi_newNodeData(mi_elemData(:,3),2) ...
    mi_newNodeData(mi_elemData(:,4),2) mi_newNodeData(mi_elemData(:,5),2) ...
    mi_newNodeData(mi_elemData(:,6),2)],2);
mi_elemCenters(:,2)= mean([mi_newNodeData(mi_elemData(:,3),3) ...
    mi_newNodeData(mi_elemData(:,4),3) mi_newNodeData(mi_elemData(:,5),3) ...
    mi_newNodeData(mi_elemData(:,6),3)],2);

% This is one square cut at centre of size s = dx(or
% dy)/(sqrt(2))
rho_up = min(1,(volFrac+DiffInDensity/2));
rho_bottom = max(volFrac - DiffInDensity/2,min_rho);
volFracBar = (volFrac-rho_bottom)/(rho_up-min_rho);
initial_mi_rho = rho_up*ones(mi_nElems,1);
%   sx = sqrt(rho_up*dx*dy*AR/(2*(rho_up+rho_bottom)));
val1 = 0.5*(AR+1+real(sqrt((AR+1)^2-4*AR*volFracBar)))/AR;
val2 =  0.5*(AR+1-real(sqrt((AR+1)^2-4*AR*volFracBar)))/AR;
if and(val1>=0,val1<=1)
    sy = 1-val1;
elseif and(val2>=0,val2<=1)
    sy = 1-val2;
else
    sy = 0.5;
end
sx = 1-(1-sy)*AR;

initial_mi_rho(and(and(mi_elemCenters(:,1)<dx/2+sx/2,mi_elemCenters(:,2)<dy/2+sy/2),and(mi_elemCenters(:,1)>dx/2-sx/2,mi_elemCenters(:,2)>dy/2-sy/2))) = rho_bottom;
finalVolFrac = sum(initial_mi_rho)/numel(initial_mi_rho);
initial_mi_rho = min(1,max(min_rho,initial_mi_rho-(finalVolFrac-volFrac)));
end
function [B0, Det] = B_Det(XX,R,S)

ZP = 1.0+R;   % zeta plus 1
EP = 1.0+S;     %eta puls 1
ZM = 1.0-R;
EM = 1.0-S;
%1. with respect to R
P=zeros(2,4);
P(1,1) = -0.25*EM;
P(1,2) = 0.25*EM;
P(1,3) = 0.25*EP;
P(1,4) = -0.25*EP;
%2. with respect to S
P(2,1) = -0.25*ZM;
P(2,2) = -0.25*ZP;
P(2,3) =0.25*ZP;
P(2,4) =0.25*ZM;
%Evaluation of the Jacobian matrix
XJ  =P*XX;
% Determinant
Det = XJ(1,1)*XJ(2,2)-XJ(1,2)*XJ(2,1);
if(Det <= 0.0)
    disp('ERROR: Jacobian matrix is not positive definite.');
    return;
end
%inverse of jacobian matrix
XJI=zeros(2,2);
XJI(1,1) = XJ(2,2)/Det;
XJI(1,2) = -XJ(1,2)/Det;
XJI(2,1) = -XJ(2,1)/Det;
XJI(2,2) = XJ(1,1)/Det;

R1=[1 0 0 0;0 0 0 1;0 1 1 0];
R2=zeros(4);
R2(1:2,1:2)=XJI;R2(3:4,3:4)=XJI;
R3=zeros(4,8);
R3(1:2,1:2:7)=P;
R3(3:4,2:2:8)=P;
B0=R1*R2*R3;
end
function [B,Det]=calc_Bdet1(Elem,Node)
Nelem=size(Elem,1);
B=cell(Nelem,4);
Det=cell(Nelem,4);
XG = [-1/sqrt(3) 1/sqrt(3)  1/sqrt(3) -1/sqrt(3) ];
YG = [-1/sqrt(3) -1/sqrt(3) 1/sqrt(3)  1/sqrt(3)];
XX=zeros(4,2);
i = 1;
XX(:,1:2) =  Node(Elem(i,3:6),2:3); %XX(:,2) =  Node(Elem(i,3:6),3);
allB = cell(4,1);
allDet = cell(4,1);
for GSP = 1:4
    RI = XG(GSP);
    SI = YG(GSP);
    [B1, Det1] = B_Det(XX,RI,SI);
    allB{i,GSP}=B1;
    allDet{i,GSP}=Det1;
end
for i=1:Nelem
    for GSP = 1:4
        B{i,GSP}=allB{1,GSP};
        Det{i,GSP}=allDet{1,GSP};
    end
end
end
function [Ke,D]=calc_elem_mat(elemData,matProp,B,Det)
YM = matProp(elemData(1,2),2);
nu = matProp(elemData(1,2),3);
t = matProp(elemData(1,2),4);
% constitutive matrix for plane-stress
D=zeros(3,3);
dummy = YM/(1-nu*nu);
D(1,1) = dummy;
D(1,2) = dummy*nu;
D(2,1) = D(1,2);
D(2,2) = D(1,1);
D(3,3) = dummy*(1-nu)/2;
Ke= Kelem_common_simple(D,t,B,Det{1,1});
end

function [F] = Felem_common(D,t,B,Det)
F = (B{1,1}+B{1,2}+B{1,3}+B{1,4})'*D*Det*t;
end
function [iK,jK,iF,jF,nNodes,nElems,B,Det,elemDofs,newNodeData,elemData,freeDofs,prscDofs,prscComp,kfLogic] =generateMicrostructureFramework(nelx,nely,Lx,Ly,applyPBC)

if nargin ==4
    applyPBC = 1;
end
nElems = nelx*nely;
nNodes = (nelx+1)*(nely+1);

nodenrs = reshape(1:(1+nelx)*(1+nely),1+nely,1+nelx);
edofVec = reshape(2*nodenrs(1:end-1,1:end-1)-1,nelx*nely,1);
edofMat = repmat(edofVec,1,8)+repmat([0 1 2*nely+[2 3 4 5] 2 3],nelx*nely,1);
iK = reshape(kron(edofMat,ones(8,1))',64*nelx*nely,1);
jK = reshape(kron(edofMat,ones(1,8))',64*nelx*nely,1);

delx = Lx/nelx;dely = Ly/nely;
NodeNrs=(1:(nelx+1)*(nely+1))';
y=reshape(repmat((0:dely:dely*nely)',1,nelx+1),(nelx+1)*(nely+1),1);
x=reshape(repmat((0:delx:delx*nelx),nely+1,1),(nelx+1)*(nely+1),1);
nodeData = [NodeNrs,x,y];
elemData = zeros(nelx*nely,6);
k = 0;
for i=1:nelx
    for j=1:nely
        k=k+1;
        elemData(k,:) =[k,1,j+(i-1)*(nely+1),j+i*(nely+1),j+1+i*(nely+1),j+1+(i-1)*(nely+1)];
    end
end
newNodeData = nodeData;
elemDofs=[2*newNodeData(elemData(:,3),1)-1  2*newNodeData(elemData(:,3),1) ...
    2*newNodeData(elemData(:,4),1)-1 2*newNodeData(elemData(:,4),1) ...
    2*newNodeData(elemData(:,5),1)-1 2*newNodeData(elemData(:,5),1) ...
    2*newNodeData(elemData(:,6),1)-1 2*newNodeData(elemData(:,6),1) ];
if applyPBC == 1
    mins(1,1)= min(min(nodeData(:,2)));
    mins(2,1)= min(min(nodeData(:,3)));
    maxs(1,1)= max(max(nodeData(:,2)));
    maxs(2,1)= max(max(nodeData(:,3)));
    
    left = nodeData(and(nodeData(:,2)==mins(1,1),and(nodeData(:,3)~=mins(2,1),nodeData(:,3)~=maxs(2,1))),1);
    right = nodeData(and(nodeData(:,2)==maxs(1,1),and(nodeData(:,3)~=mins(2,1),nodeData(:,3)~=maxs(2,1))),1);
    bottom  = nodeData(and(nodeData(:,3)==mins(2,1),and(nodeData(:,2)~=mins(1,1),nodeData(:,2)~=maxs(1,1))),1);
    top= nodeData(and(nodeData(:,3)==maxs(2,1),and(nodeData(:,2)~=mins(1,1),nodeData(:,2)~=maxs(1,1))),1);
    
    c1 = nodeData(and(nodeData(:,2)==mins(1,1),nodeData(:,3)==mins(2,1)),1);
    c2 = nodeData(and(nodeData(:,2)==maxs(1,1),nodeData(:,3)==mins(2,1)),1);
    c3 = nodeData(and(nodeData(:,2)==maxs(1,1),nodeData(:,3)==maxs(2,1)),1);
    c4 = nodeData(and(nodeData(:,2)==mins(1,1),nodeData(:,3)==maxs(2,1)),1);
    
    allDofs = 1:2*nNodes;
    prsc0 = reshape([2*[c1,c2,c3,c4]-1;2*[c1,c2,c3,c4]],[],1);
    
    rightNtop = [right*2-1;right*2;top*2-1;top*2];
    prscDofs = [prsc0;rightNtop];
    freeDofs = setdiff(allDofs,prscDofs);
    leftNbottom = [left*2-1;left*2;bottom*2-1;bottom*2];
    prscComp = [repmat([1;2],4,1);leftNbottom];
    % lefts = [left*2-1;left*2];
    % bottoms = [bottom*2-1;bottom*2];
    iF = reshape(repmat(elemDofs,1,3)',[],1);
    jF = repmat(reshape(repmat(1:3,8,1),24,1),nElems,1);
    
    jKnew = jK;
    iKnew = iK;
    iFnew = iF;
    for j = 1:length(prscDofs)
        logicArray = (jK == prscDofs(j));
        jKnew(logicArray) = prscComp(j);
    end
    for i = 1:length(prscDofs)
        logicArray = (iK == prscDofs(i));
        iKnew(logicArray) = prscComp(i);
    end
    for i = 1:length(prscDofs)
        logicArray = (iF == prscDofs(i));
        iFnew(logicArray) = prscComp(i);
    end
    indexMap(freeDofs) = 1:length(freeDofs);
    fLogic = and(iFnew~=1,iFnew~=2);
    kLogic = and(and(jKnew~=1,jKnew~=2),and(iKnew~=1,iKnew~=2));
    iK = indexMap(iKnew(kLogic));
    jK = indexMap(jKnew(kLogic));
    iF = indexMap(iFnew(fLogic));
    jF = jF(fLogic);
    kfLogic = {kLogic,fLogic};
else
    iF = [];
    jF = [];
    freeDofs = [];
    prscDofs = [];
    prscComp = [];
    kfLogic = [];
    
end
[B,Det]= getB_Det_one(delx,dely);
end
function strainCell=get_strain_for_SIMP(u,B,elemList,elemDOFs)
noOfComputations=size(u,2);
strainCell=cell(noOfComputations,1);
nElem = length(elemList);
for i=1:noOfComputations
    strainCell{i,1}=zeros(3,nElem);
end
counter = 1;
for elem = elemList
    curr_Dofs=elemDOFs(elem,:);
    for j =1:noOfComputations
        strainCell{j,1}(:,counter)=B* u(curr_Dofs,j);
    end
    counter = counter + 1;
end
end

function [B,Det,B0,Det0]=getB_Det_one(delx,dely)
B = cell(1,4);
Det = cell(1,4);
XG = [-1/sqrt(3) 1/sqrt(3)  1/sqrt(3) -1/sqrt(3) ];
YG = [-1/sqrt(3) -1/sqrt(3) 1/sqrt(3)  1/sqrt(3)];
XX=zeros(4,2);
xs = [0 delx delx 0];ys = [ 0 0 dely dely];
XX(:,1) =  xs; XX(:,2) =  ys;

for GSP = 1:4
    RI = XG(GSP);
    SI = YG(GSP);
    [B1, Det1] = B_Det(XX,RI,SI);
    B{1,GSP} = B1;
    Det{1,GSP} = Det1;
end
if nargout>2
    [B0, Det0] = B_Det(XX,0,0);
end
end
function [D_H,D_H_Grad] = getHomogenizedDandGradient(u,nElems,D,b,Det,thickness,elemDofs,rho,V_elem,p)
numOfMS = size(rho,2);
D_H = zeros(3,3,numOfMS);
Us = zeros(8,nElems,3,numOfMS);
parfor MS = 1:numOfMS
    for k = 1:3
        Us(:,:,k,MS) = reshape(u(reshape(elemDofs',[],1),k,MS),8,nElems);
    end
end
parfor MS = 1:numOfMS
    for k = 1:3
        for i = 1:4
            D_H(:,k,MS) = D_H(:,k,MS) +sum(bsxfun(@times,(rho(:,MS).^p)',bsxfun(@minus,eq((1:3)',k),b{1,i}*Us(:,:,k,MS)))*Det{1,i},2);
        end
        
    end
end
for MS = 1:numOfMS
    D_H(:,:,MS) = D*D_H(:,:,MS);
end
D_H = D_H*thickness/(nElems*V_elem);
if nargout>=2
    D_H_Grad = zeros(nElems,3,3,numOfMS);
    parfor MS = 1:numOfMS
        for j = 1:3
            for k = 1:3
                for i = 1:4
                    D_H_Grad(:,j,k,MS)= D_H_Grad(:,j,k,MS) + p*(rho(:,MS).^(p-1)).*sum((bsxfun(@minus,eq((1:3)',j),b{1,i}*Us(:,:,j,MS))'*D).*bsxfun(@minus,eq((1:3)',k),b{1,i}*Us(:,:,k,MS))',2)*Det{1,i};
                end
            end
        end
    end
    D_H_Grad = permute(D_H_Grad,[2,3,1,4]);
    D_H_Grad = D_H_Grad *thickness/(nElems*V_elem);
end
end
function mi_wgt = getPeriodicDomainWeight(mi_dx,mi_dy,mi_nElems,mi_nely,elemArea,radius_as_numOfElem)
% This function assumes mi_dx = mi_dy
distances = zeros(1,(ceil(radius_as_numOfElem)*2+1)^2);
indices = zeros(1,(ceil(radius_as_numOfElem)*2+1)^2);
ctr = 2;
mi_wgt = zeros(mi_nElems,mi_nElems);
indicesIndexInRow = zeros(1+2*ceil(radius_as_numOfElem));
indicesInRow = zeros(1+2*ceil(radius_as_numOfElem));
rowIndexCtr = ones(1+2*ceil(radius_as_numOfElem),1);
centerRow = ceil(radius_as_numOfElem)+1;
indicesIndexInRow (centerRow,rowIndexCtr(centerRow,1)) = 1;
indicesInRow(centerRow,rowIndexCtr(centerRow,1)) = 0;
rowIndexCtr(centerRow,1) = 2;
for row = 2:ceil(radius_as_numOfElem)+1
    template_dist_one_fourth = sqrt(((row-1)*mi_dy)^2+((-(row-1):(row-2))*mi_dx).^2);
    template_dist = repmat(template_dist_one_fourth,1,4);
    template_idx = [-(row-1)*mi_nely-(row-1):mi_nely:(row-2)*mi_nely-(row-1),(row-1)*mi_nely-(row-1):(row-1)*mi_nely+(row-2),...
        (row-1)*mi_nely+(row-1):-mi_nely:-(row-2)*mi_nely+(row-1),-(row-1)*mi_nely+(row-1):-1:-(row-1)*mi_nely-(row-2)];
    distances(1,ctr:ctr+(row-1)*2*4-1) = template_dist;
    indices(1,ctr:ctr+(row-1)*2*4-1) = template_idx;
    
    indicesInRow(centerRow-(row-1),rowIndexCtr(centerRow-(row-1),1):rowIndexCtr(centerRow-(row-1),1)+(row-1)*2) = -(row-1)*mi_nely-(row-1):mi_nely:(row-1)*mi_nely-(row-1);
    indicesIndexInRow(centerRow-(row-1),rowIndexCtr(centerRow-(row-1),1):rowIndexCtr(centerRow-(row-1),1)+(row-1)*2) = ctr:ctr+(row-1)*2;
    rowIndexCtr(centerRow-(row-1),1) = rowIndexCtr(centerRow-(row-1),1) +(row-1)*2+1;
    indicesInRow(centerRow+(row-1),rowIndexCtr(centerRow+(row-1),1):rowIndexCtr(centerRow+(row-1),1)+(row-1)*2) = -(row-1)*mi_nely+(row-1):mi_nely:(row-1)*mi_nely+(row-1);
    indicesIndexInRow(centerRow+(row-1),rowIndexCtr(centerRow+(row-1),1):rowIndexCtr(centerRow+(row-1),1)+(row-1)*2) = ctr+(row-1)*4:ctr+(row-1)*6;
    rowIndexCtr(centerRow+(row-1),1) = rowIndexCtr(centerRow+(row-1),1) +(row-1)*2+1;
    for row2 = (centerRow-(row-2):centerRow+(row-2))
        idx = row2-(centerRow-(row-1));
        indicesInRow(row2,rowIndexCtr(row2,1):rowIndexCtr(row2,1)+1) = [-(row-1)*mi_nely-(row-1),(row-1)*mi_nely-(row-1)]+idx;
        indicesIndexInRow(row2,rowIndexCtr(row2,1):rowIndexCtr(row2,1)+1) = [ctr+(row-1)*2+idx,ctr+(row-1)*2*4-idx];
        rowIndexCtr(row2,1) = rowIndexCtr(row2,1)+2;
        
    end
    ctr = ctr+(row-1)*2*4;
end
rad = sqrt(elemArea)*radius_as_numOfElem;
HUGE_NUMBER = 1e3;

for elem=1:mi_nElems
    dist = HUGE_NUMBER*ones(mi_nElems,1);
    myIndices = indices;
    row = rem(elem-1,mi_nely)+1;
    myIndices(indicesIndexInRow(centerRow+mi_nely-row+1:end,:)) =  myIndices(indicesIndexInRow(centerRow+mi_nely-row+1:end,:)) - mi_nely;
    myIndices(indicesIndexInRow(1:centerRow-row,:)) =  myIndices(indicesIndexInRow(1:centerRow-row,:)) + mi_nely;
    
    elemIndices = elem + myIndices;
    elemIndices(elemIndices<=0) = elemIndices(elemIndices<=0)+mi_nElems;
    elemIndices(elemIndices>mi_nElems) = elemIndices(elemIndices>mi_nElems)-mi_nElems;
    dist(elemIndices) = distances;
    mi_wgt(elem,:)=(rad-dist).*le(dist,rad);
    mi_wgt(elem,:)=mi_wgt(elem,:)/sum(mi_wgt(elem,:));
end
mi_wgt=sparse(mi_wgt);
end
function [N,dN] = getShapeFunctionAndItsDerivative(xi,eta)
%For each xi,eta combination, dN contains two consecutive columns for
%dN/dxi, dN/deta
%Therefore, dN returns size(xi or eta,1) times 2 columns and 4 rows (for
%four dof)
xis = [-1,1,1,-1]';
etas = [-1,-1,1,1]';

N = 0.25*(1+xis*xi').*(1+etas*eta');
dN1 = 0.25*(xis*ones(size(xi'))).*(1+etas*eta');
dN2 = 0.25*(1+xis*xi').*(etas*ones(size(eta')));
dN(1:4,1:2:length(xi)*2) = dN1;
dN(1:4,2:2:length(xi)*2) = dN2;
end
function interiorU = interpolateValuesInTheInterior(nodalU,interiorX,interiorY,nodalX,nodalY)
% nodalXi = [-1,1,1,-1];
% nodalEta = [-1,-1,1,1];
if nargin<=3
    nodalX = [-1,1];
    nodalY = [-1,1];
end
xiInterior = (2*interiorX-(max(nodalX)+min(nodalX)))/(max(nodalX)-min(nodalX));
etaInterior = (2*interiorY-(max(nodalY)+min(nodalY)))/(max(nodalY)-min(nodalY));

N1 = 0.25*(1-xiInterior).*(1-etaInterior);
N2 = 0.25*(1+xiInterior).*(1-etaInterior);
N3 = 0.25*(1+xiInterior).*(1+etaInterior);
N4 = 0.25*(1-xiInterior).*(1+etaInterior);

interiorU = N1*nodalU(1) + N2*nodalU(2) + N3*nodalU(3) + N4*nodalU(4);
end
function [K] = Kelem_common_simple(D,t,B,Det)
WT = 1*t*Det;
K = (B{1,1}'*D*B{1,1} + B{1,2}'*D*B{1,2} + B{1,3}'*D*B{1,3} + B{1,4}'*D*B{1,4})*WT;
end
function [elemdata,nodedata,matprop,dispbc,forcebc,nNode,nElem,dof] = readData( fileName,dof )
fid = fopen(fileName,'r');
fgets(fid);
fgets(fid);
nElem = fscanf(fid,'%i\n',1);
fgets(fid);
fgets(fid);
elemdata = fscanf(fid,'%i\n',[6,nElem]);
elemdata = elemdata';
% Material property data
fgets(fid);
fgets(fid);
NmaterialID = 1;
for i = 1:nElem
    if(NmaterialID < elemdata(i,2))
        NmaterialID = elemdata(i,2);
    end
end
matprop = fscanf(fid,'%g\n',[4,NmaterialID]);
matprop = matprop';
% Nodal coordinates
fgets(fid);
nNode = fscanf(fid,'%i\n',1);
fgets(fid);
fgets(fid);
nodedata = fscanf(fid,'%i %g %g\n',[dof+1, nNode]);
nodedata = nodedata';
% Displacement boundary conditions
fgets(fid);
Ndispbc = fscanf(fid,'%i\n',1);
fgets(fid);
dispbc = fscanf(fid,'%i %g %g\n',[3, Ndispbc]);
dispbc = dispbc';
% Force boundary conditions
fgets(fid);
Nforcebc = fscanf(fid,'%i\n',1);
fgets(fid);
forcebc = fscanf(fid,'%i %g %g\n',[3, Nforcebc]);
forcebc = forcebc';
fclose(fid);
end
function [objective,gradients_wrt_mi_rho,u,grad,gradients_wrt_theta] = solLinElastWithGlobalGrad_MicroStructRotatEffi(D_homo,Ke_unit,newNodeData,t,rho_vol,theta,freeDofs,forceBC,elemDofs,nNode,B,Det,grad_D,zones,Q,iQ,jQ)
%type 1: Strain Energy(mean Compliance) minimization

rotOn = ~(length(unique(theta))==1 && theta(1)==0);
noOfMS = size(D_homo,3);
mi_nElems = size(grad_D,3);
zoneElems = cell2mat(zones);
nElems = size(zoneElems,1);
Ke = zeros(8,8,nElems);
B_common = B(1,:);
Det_common = Det{1,1};
elemZones = zeros(nElems,1);
for group = 1:size(zones,1)
    elemZones(zones{group,1},1) = group;
end
if rotOn
    iD = reshape(D_homo(:,:,elemZones),[],1);
    D_homo_big = sparse(iQ,jQ,iD);
    D_elem_big = Q*D_homo_big*Q';
    linearInd = sub2ind(size(D_elem_big),iQ,jQ);
    D_elem_lin = reshape(full(D_elem_big(linearInd)),9,1,[]);
    Ke = bsxfun(@times,Ke_unit(:,:,1),D_elem_lin(1,1,:))+bsxfun(@times,Ke_unit(:,:,2),D_elem_lin(5,1,:))+...
        bsxfun(@times,Ke_unit(:,:,3),D_elem_lin(9,1,:))+bsxfun(@times,Ke_unit(:,:,4),D_elem_lin(4,1,:))+...
        bsxfun(@times,Ke_unit(:,:,5),D_elem_lin(7,1,:))+bsxfun(@times,Ke_unit(:,:,6),D_elem_lin(8,1,:));
else
    for MS = 1:noOfMS
        Ke(:,:,zones{MS,1}) = repmat(Ke_unit(:,:,1)*D_homo(1,1,MS)+Ke_unit(:,:,2)*D_homo(2,2,MS)+Ke_unit(:,:,3)*D_homo(3,3,MS)+...
            Ke_unit(:,:,4)*D_homo(1,2,MS)+Ke_unit(:,:,5)*D_homo(1,3,MS)+Ke_unit(:,:,6)*D_homo(2,3,MS),1,1,numel(zones{MS,1}));
    end
end
iK = reshape(kron(elemDofs',ones(8,1)),64*nElems,1);
jK = reshape(kron(elemDofs',ones(1,8)),64*nElems,1);
sK = Ke(:);
K = sparse(iK,jK,sK,2*nNode,2*nNode);
K = (K+K')/2;
u = zeros(2*nNode,1);
F = zeros(2*nNode,1);
fdof = (newNodeData(forceBC(:,1),1)-1)*2 + forceBC(:,2);
F(fdof)=forceBC(:,3);
u(freeDofs,1)=K(freeDofs,freeDofs)\F(freeDofs);
objective = u'*K*u;
if nargout>=2
    if nargout>=5
        grad_Ke_wrt_theta = zeros(8,8,nElems);
    end
    if rotOn
        linearInd = sub2ind([3*nElems,3*nElems],iQ,jQ);
        D_elem_lin = zeros(9,1,mi_nElems,nElems);
        parfor mi_elem = 1:mi_nElems
            
            iD = reshape(grad_D(:,:,mi_elem,elemZones),[],1);
            D_homo_big = sparse(iQ,jQ,iD);
            D_elem_big = Q*D_homo_big*Q';
            D_elem_lin(:,1,mi_elem,:) = reshape(full(D_elem_big(linearInd)),9,1,1,[]);
        end
        grad_Ke_wrt_mi_rho = bsxfun(@times,Ke_unit(:,:,1),D_elem_lin(1,1,:,:))+bsxfun(@times,Ke_unit(:,:,2),D_elem_lin(5,1,:,:))+...
            bsxfun(@times,Ke_unit(:,:,3),D_elem_lin(9,1,:,:))+bsxfun(@times,Ke_unit(:,:,4),D_elem_lin(4,1,:,:))+...
            bsxfun(@times,Ke_unit(:,:,5),D_elem_lin(7,1,:,:))+bsxfun(@times,Ke_unit(:,:,6),D_elem_lin(8,1,:,:));
    else
        grad_Ke_wrt_mi_rho_per_zone = zeros(8,8,mi_nElems);
        grad_Ke_wrt_mi_rho = zeros(8,8,mi_nElems,nElems);
        for MS = 1:noOfMS
            parfor mi_elem = 1:mi_nElems
                grad_Ke_wrt_mi_rho_per_zone (:,:,mi_elem) = Ke_unit(:,:,1)*grad_D(1,1,mi_elem,MS)...
                    +Ke_unit(:,:,2)*grad_D(2,2,mi_elem,MS)+Ke_unit(:,:,3)*grad_D(3,3,mi_elem,MS)+...
                    Ke_unit(:,:,4)*grad_D(1,2,mi_elem,MS)+Ke_unit(:,:,5)*grad_D(1,3,mi_elem,MS)+Ke_unit(:,:,6)*grad_D(2,3,mi_elem,MS);
            end
            grad_Ke_wrt_mi_rho(:,:,:,zones{MS,1}) = repmat(grad_Ke_wrt_mi_rho_per_zone,1,1,1,numel(zones{MS,1}));
        end
    end
    if nargout>=4
        uelem = u(elemDofs);
        grad= -sum(squeeze(sum(repmat(reshape(uelem',8,1,nElems),1,8,1).*Ke,1))'.*uelem,2)./(rho_vol.^(1));
    end
    if nargout>=5
        parfor  elem = 1:nElems
            ct = cos(theta(elem));
            st = sin(theta(elem));
            c2t = cos(2*theta(elem));
            s2t = sin(2*theta(elem));
            T = [ct^2 st^2 s2t;st^2 ct^2 -s2t;-s2t/2 s2t/2 c2t];
            
            dT = [-s2t s2t 2*c2t;s2t -s2t -2*c2t;-c2t c2t -2*s2t];
            D_elem = dT*D_homo(:,:,elemZones(elem))*T';
            D_elem = D_elem + D_elem';
            grad_Ke_wrt_theta(:,:,elem) = Kelem_common_simple(D_elem,t,B_common,Det_common);
        end
    end
    gradients_wrt_mi_rho = zeros(mi_nElems,nElems);
    gradients_wrt_theta = 0;
    
    parfor elem = 1:nElems
        uelem = u(elemDofs(elem,:));
        gradients_wrt_mi_rho(:,elem) = -squeeze(sum(repmat(uelem,1,8,mi_nElems).*grad_Ke_wrt_mi_rho(:,:,:,elem),1))'*uelem;
    end
    if nargout>=5
        uelem = u(elemDofs);
        gradients_wrt_theta= -sum(squeeze(sum(repmat(reshape(uelem',8,1,nElems),1,8,1).*grad_Ke_wrt_theta,1))'.*uelem,2);
    end
end
end
function [u,obj,grad] = solveLinearElasticityWithGradEfficient(Ke,newNodeData,rho_vol,p,freeDofs,forceBC,elemDofs,nNode,iK,jK,interpolation)
nElems = size(elemDofs,1);
switch interpolation
    case 1
        sK = reshape(Ke(:)*(rho_vol.^p)',64*nElems,1);
    case 2
        q = p;
        sK = reshape(Ke(:)*(rho_vol./(1+q*(1-rho_vol)))',64*nElems,1);
end
K = sparse(iK,jK,sK,2*nNode,2*nNode);
K = (K+K')/2;
u = zeros(2*nNode,1);
fdof = (newNodeData(forceBC(:,1),1)-1)*2 + forceBC(:,2);
F=accumarray(fdof,forceBC(:,3),[2*nNode,1]);

u(freeDofs,1)=K(freeDofs,freeDofs)\F(freeDofs);

if nargout>1
    compliance = u'*K*u;
    switch interpolation
        case 1
            grad = -p*rho_vol.^(p-1).*sum((u(elemDofs)*Ke).*u(elemDofs),2);
        case 2
            q = p;
            grad = -((1+q)./((1+q*(1-rho_vol)).^2)).*sum((u(elemDofs)*Ke).*u(elemDofs),2);
    end
    obj = compliance;
end
end
function [u] = solveMicrostructureFEA_efficient(rho,D,iK,jK,iF,jF,nNodes,t,p,B,Det,freeDofs,prscDofs,prscComp,kfLogic)
[nElems,NumOfMS] = size(rho);
Ke = Kelem_common_simple(D,t,B,Det{1,1});
Fe = Felem_common(D,t,B,Det{1,1});
totDofs = 2*nNodes;
uFree = zeros(length(freeDofs),3,NumOfMS);
u = zeros(totDofs,3,NumOfMS);
parfor MS =1:NumOfMS
    sK = reshape(reshape(Ke,64,1)*(rho(:,MS)'.^p),64*nElems,1);
    K = sparse(iK,jK,sK(kfLogic{1,1}));
    sF = reshape(reshape(Fe,24,1)*(rho(:,MS)'.^p),8*3*nElems,1);
    F = sparse(iF,jF,sF(kfLogic{1,2}));
    uFree(:,:,MS) = full(K\F);
end
u(freeDofs,:,:) = uFree;
u(prscDofs,:,:) = u(prscComp,:,:);
end
function [x,change,lmid] = updateRho(x,dc,volfrac,totVolume,min_rho)
%% OPTIMALITY CRITERIA UPDATE OF DESIGN VARIABLES AND PHYSICAL DENSITIES
cutoff = 1e-8;
volPerElem = 1;
if nargin<5
    min_rho = 1e-3;
end

l1 = 0; l2 = 1e10; move = 0.05;
if(max(max(dc))>0)
    grads = dc-1.1*max(max(dc));
else
    grads = dc;
end
vars = x;
while (l2-l1)/(l1+l2) > cutoff
    lmid = 0.5*(l2+l1);
    varsnew = max(min_rho,max(vars-move,min(1,min(vars...
        +move,vars.*(-(grads./volPerElem)/lmid).^0.5))));
    if sum(sum(varsnew.*volPerElem)) > volfrac*totVolume
        l1 = lmid;
    else
        l2 = lmid;
    end
end
xnew = varsnew;
change = max(abs(xnew(:)-x(:)));
x = xnew;
end
function [x,change,lmid] = updateRho1(x,dc,volfrac,totVolume,min_rho,volPerElem,cutoff,move,preserveBoundary,dampCoeff,volConst)
if nargin <= 8
    cutoff = 1e-10;
end
if nargin <=9
    move = 0.2;
end
if nargin <= 11
    dampCoeff = 0.5;
end
nE_MS = size(x,1);
sizeMS = round(sqrt(nE_MS));

l1 = 0; l2 = 1e10;
if(max(max(dc))>0)
    grads = dc-1.1*max(max(dc));
else
    grads = dc;
end
grads = grads/abs(min(min(grads)));
vars = x;
if preserveBoundary
    fixedIndices = unique([1:sizeMS,1:sizeMS:sizeMS*sizeMS,sizeMS:sizeMS:sizeMS*sizeMS,(sizeMS*(sizeMS-1)+1):sizeMS*sizeMS]);
    allIndices = 1:size(vars,1);
    freeIndices = setdiff(allIndices,fixedIndices);
end
while (l2-l1)/(l1+l2) > cutoff
    lmid = 0.5*(l2+l1);
    varsNew = max(min_rho,max(vars-move,min(1,min(vars...
        +move,vars.*(-(grads./volPerElem)/lmid).^dampCoeff))));%
    
    if preserveBoundary
        allNotMins = any(varsNew(freeIndices,:)>50*min_rho,1);
        varsNew(fixedIndices,allNotMins) = 1;
    end
    if sum(sum(varsNew.*volPerElem)) > volfrac*totVolume
        l1 = lmid;
    else
        l2 = lmid;
    end
end
xnew = varsNew;
change = max(abs(xnew(:)-x(:)));
x = xnew;
end